import { ConnectionOptions } from "../connection/ConnectionOptions";
import { ColumnMetadata } from "../metadata/ColumnMetadata";
/**
 * Provides base functionality for all driver implementations.
 */
export declare abstract class BaseDriver {
    abstract connectionOptions: ConnectionOptions;
    private transactionActive;
    protected abstract checkIfConnectionSet(): void;
    /**
     * Executes a given SQL query and returns raw database results.
     */
    abstract query<T>(query: string): Promise<T>;
    /**
     * Escapes given value.
     */
    abstract escape(value: any): any;
    /**
     * Updates rows that match given conditions in the given table.
     */
    update(tableName: string, valuesMap: Object, conditions: Object): Promise<void>;
    private escapeObjectMap(objectMap);
    /**
     * Insert a new row into given table.
     */
    insert(tableName: string, keyValues: Object): Promise<any>;
    /**
     * Deletes from the given table by a given conditions.
     */
    delete(tableName: string, conditions: Object): Promise<void>;
    /**
     * Starts transaction.
     */
    beginTransaction(): Promise<void>;
    /**
     * Commits transaction.
     */
    commitTransaction(): Promise<void>;
    /**
     * Rollbacks transaction.
     */
    rollbackTransaction(): Promise<void>;
    /**
     * Checks if transaction is active.
     */
    isTransactionActive(): boolean;
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    preparePersistentValue(value: any, column: ColumnMetadata): any;
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    prepareHydratedValue(value: any, column: ColumnMetadata): any;
    /**
     * Inserts rows into closure table.
     */
    insertIntoClosureTable(tableName: string, newEntityId: any, parentId: any, hasLevel: boolean): Promise<number>;
    protected logQuery(query: string): void;
    protected logQueryError(error: any): void;
    protected logFailedQuery(query: string): void;
    protected log(message: any, level: "log" | "debug" | "info" | "error"): void;
}
