import { Connection } from "../connection/Connection";
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { FindOptions } from "../repository/FindOptions";
import { Repository } from "../repository/Repository";
import { ConstructorFunction } from "../common/ConstructorFunction";
import { ReactiveRepository } from "../repository/ReactiveRepository";
import { TreeRepository } from "../repository/TreeRepository";
import { ObjectLiteral } from "../common/ObjectLiteral";
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its methods, whatever
 * entity type are you passing.
 */
export declare class EntityManager {
    private connection;
    constructor(connection: Connection);
    /**
     * Gets repository for the given entity class.
     */
    getRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityClass: string): Repository<Entity>;
    /**
     * Gets repository for the given entity class or name.
     */
    getRepository<Entity>(entityClassOrName: ConstructorFunction<Entity> | Function | string): Repository<Entity>;
    /**
     * Gets repository for the given entity class.
     */
    getTreeRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): TreeRepository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getTreeRepository<Entity>(entityClass: string): TreeRepository<Entity>;
    /**
     * Gets reactive repository of the given entity.
     */
    getReactiveRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): ReactiveRepository<Entity>;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Function): boolean;
    hasId(target: Function | string, entity: Function): boolean;
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    createQueryBuilder<Entity>(entityClass: ConstructorFunction<Entity> | Function, alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create<Entity>(entityClass: ConstructorFunction<Entity> | Function, fromRawEntity?: Object): Entity;
    /**
     * Creates a entities from the given array of plain javascript objects.
     */
    createMany<Entity>(entityClass: ConstructorFunction<Entity> | Function, copyFromObjects: any[]): Entity[];
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize<Entity>(entityClass: ConstructorFunction<Entity> | Function, object: Object): Promise<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge<Entity>(entityClass: ConstructorFunction<Entity> | Function, ...objects: ObjectLiteral[]): Entity;
    /**
     * Persists (saves) a given entity in the database.
     */
    persist<Entity>(entity: Entity): Promise<Entity>;
    persist<Entity>(targetOrEntity: Function | string, entity: Entity): Promise<Entity>;
    /**
     * Removes a given entity from the database.
     */
    remove<Entity>(entity: Entity): Promise<Entity>;
    remove<Entity>(targetOrEntity: Function | string, entity: Entity): Promise<Entity>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<[Entity[], number]>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, options: FindOptions): Promise<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne<Entity>(entityClass: ConstructorFunction<Entity> | Function, conditions: Object, options: FindOptions): Promise<Entity>;
    /**
     * Finds entity with given id.
     */
    findOneById<Entity>(entityClass: ConstructorFunction<Entity> | Function, id: any, options?: FindOptions): Promise<Entity>;
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query: string): Promise<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any>): Promise<any>;
    /**
     * Sets given relatedEntityId to the value of the relation of the entity with entityId id.
     * Should be used when you want quickly and efficiently set a relation (for many-to-one and one-to-many) to some entity.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    setRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: string, entityId: any, relatedEntityId: any): Promise<void>;
    setRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: ((t: Entity) => string | any), entityId: any, relatedEntityId: any): Promise<void>;
    /**
     * Adds a new relation between two entities into relation's many-to-many table.
     * Should be used when you want quickly and efficiently add a relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    addToRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: string, entityId: any, relatedEntityIds: any[]): Promise<void>;
    addToRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: ((t: Entity) => string | any), entityId: any, relatedEntityIds: any[]): Promise<void>;
    /**
     * Removes a relation between two entities from relation's many-to-many table.
     * Should be used when you want quickly and efficiently remove a many-to-many relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeFromRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: string, entityId: any, relatedEntityIds: any[]): Promise<void>;
    removeFromRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relationName: ((t: Entity) => string | any), entityId: any, relatedEntityIds: any[]): Promise<void>;
    /**
     * Performs both #addToRelation and #removeFromRelation operations.
     * Should be used when you want quickly and efficiently and and remove a many-to-many relation between two entities.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    addAndRemoveFromRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relation: string, entityId: any, addRelatedEntityIds: any[], removeRelatedEntityIds: any[]): Promise<void>;
    addAndRemoveFromRelation<Entity>(entityClass: ConstructorFunction<Entity> | Function, relation: ((t: Entity) => string | any), entityId: any, addRelatedEntityIds: any[], removeRelatedEntityIds: any[]): Promise<void>;
    /**
     * Removes entity with the given id.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeById<Entity>(entityClass: ConstructorFunction<Entity> | Function, id: any): Promise<void>;
    /**
     * Removes all entities with the given ids.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeByIds<Entity>(entityClass: ConstructorFunction<Entity> | Function, ids: any[]): Promise<void>;
    /**
     * Roots are entities that have no ancestors. Finds them all.
     */
    findRoots<Entity>(entityClass: ConstructorFunction<Entity> | Function): Promise<Entity[]>;
    /**
     * Creates a query builder used to get descendants of the entities in a tree.
     */
    createDescendantsQueryBuilder<Entity>(entityClass: ConstructorFunction<Entity> | Function, alias: string, closureTableAlias: string, entity: Entity): QueryBuilder<Entity>;
    /**
     * Gets all children (descendants) of the given entity. Returns them all in a flat array.
     */
    findDescendants<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<Entity[]>;
    /**
     * Gets all children (descendants) of the given entity. Returns them in a tree - nested into each other.
     */
    findDescendantsTree<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<Entity>;
    /**
     * Gets number of descendants of the entity.
     */
    countDescendants<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<number>;
    /**
     * Creates a query builder used to get ancestors of the entities in the tree.
     */
    createAncestorsQueryBuilder<Entity>(entityClass: ConstructorFunction<Entity> | Function, alias: string, closureTableAlias: string, entity: Entity): QueryBuilder<Entity>;
    /**
     * Gets all parents (ancestors) of the given entity. Returns them all in a flat array.
     */
    findAncestors<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<Entity[]>;
    /**
     * Gets all parents (ancestors) of the given entity. Returns them in a tree - nested into each other.
     */
    findAncestorsTree<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<Entity>;
    /**
     * Gets number of ancestors of the entity.
     */
    countAncestors<Entity>(entityClass: ConstructorFunction<Entity> | Function, entity: Entity): Promise<number>;
}
