import { TableMetadata } from "./TableMetadata";
import { ColumnMetadata } from "./ColumnMetadata";
import { RelationMetadata, PropertyTypeInFunction } from "./RelationMetadata";
import { IndexMetadata } from "./IndexMetadata";
import { ForeignKeyMetadata } from "./ForeignKeyMetadata";
import { NamingStrategyInterface } from "../naming-strategy/NamingStrategyInterface";
import { EntityMetadataArgs } from "../metadata-args/EntityMetadataArgs";
import { EmbeddedMetadata } from "./EmbeddedMetadata";
import { ObjectLiteral } from "../common/ObjectLiteral";
import { LazyRelationsWrapper } from "../repository/LazyRelationsWrapper";
/**
 * Contains all entity metadata.
 */
export declare class EntityMetadata {
    private lazyRelationsWrapper;
    /**
     * If entity's table is a closure-typed table, then this entity will have a closure junction table metadata.
     */
    closureJunctionTable: EntityMetadata;
    /**
     * Naming strategy used to generate and normalize names.
     */
    readonly namingStrategy: NamingStrategyInterface;
    /**
     * Entity's table metadata.
     */
    readonly table: TableMetadata;
    /**
     * Entity's relation metadatas.
     */
    readonly relations: RelationMetadata[];
    /**
     * Entity's index metadatas.
     */
    readonly indices: IndexMetadata[];
    /**
     * Entity's foreign key metadatas.
     */
    readonly foreignKeys: ForeignKeyMetadata[];
    /**
     * Entity's embedded metadatas.
     */
    readonly embeddeds: EmbeddedMetadata[];
    /**
     * Entity's column metadatas.
     */
    private readonly _columns;
    constructor(lazyRelationsWrapper: LazyRelationsWrapper, args: EntityMetadataArgs);
    /**
     * Entity's name. Equal to entity target class's name if target is set to table, or equals to table name if its set.
     */
    readonly name: string;
    /**
     * All columns of the entity, including columns that are coming from the embeddeds of this entity.
     */
    readonly columns: ColumnMetadata[];
    /**
     * Target class to which this entity metadata is bind.
     */
    readonly target: Function | string;
    /**
     * Gets the name of the target.
     */
    readonly targetName: string;
    /**
     * Checks if entity has a primary column. All user's entities must have a primary column.
     * Special entity metadatas like for junction tables and closure junction tables don't have a primary column.
     */
    readonly hasPrimaryColumn: boolean;
    /**
     * Gets the primary column.
     */
    readonly primaryColumn: ColumnMetadata;
    /**
     * Checks if entity has a create date column.
     */
    readonly hasCreateDateColumn: boolean;
    /**
     * Gets entity column which contains a create date value.
     */
    readonly createDateColumn: ColumnMetadata;
    /**
     * Checks if entity has an update date column.
     */
    readonly hasUpdateDateColumn: boolean;
    /**
     * Gets entity column which contains an update date value.
     */
    readonly updateDateColumn: ColumnMetadata;
    /**
     * Checks if entity has a version column.
     */
    readonly hasVersionColumn: boolean;
    /**
     * Gets entity column which contains an entity version.
     */
    readonly versionColumn: ColumnMetadata;
    /**
     * Checks if entity has a tree level column.
     */
    readonly hasTreeLevelColumn: boolean;
    readonly treeLevelColumn: ColumnMetadata;
    /**
     * Gets single (values of which does not contain arrays) relations.
     */
    readonly singleValueRelations: RelationMetadata[];
    /**
     * Gets single (values of which does not contain arrays) relations.
     */
    readonly multiValueRelations: RelationMetadata[];
    /**
     * Gets only one-to-one relations of the entity.
     */
    readonly oneToOneRelations: RelationMetadata[];
    /**
     * Gets only owner one-to-one relations of the entity.
     */
    readonly ownerOneToOneRelations: RelationMetadata[];
    /**
     * Gets only one-to-many relations of the entity.
     */
    readonly oneToManyRelations: RelationMetadata[];
    /**
     * Gets only many-to-one relations of the entity.
     */
    readonly manyToOneRelations: RelationMetadata[];
    /**
     * Gets only many-to-many relations of the entity.
     */
    readonly manyToManyRelations: RelationMetadata[];
    /**
     * Gets only owner many-to-many relations of the entity.
     */
    readonly ownerManyToManyRelations: RelationMetadata[];
    /**
     * Gets only owner one-to-one and many-to-one relations.
     */
    readonly relationsWithJoinColumns: RelationMetadata[];
    /**
     * Checks if there is a tree parent relation. Used only in tree-tables.
     */
    readonly hasTreeParentRelation: boolean;
    /**
     * Tree parent relation. Used only in tree-tables.
     */
    readonly treeParentRelation: RelationMetadata;
    /**
     * Checks if there is a tree children relation. Used only in tree-tables.
     */
    readonly hasTreeChildrenRelation: boolean;
    /**
     * Tree children relation. Used only in tree-tables.
     */
    readonly treeChildrenRelation: RelationMetadata;
    /**
     * Creates a new entity.
     */
    create(): any;
    /**
     * Creates an object - map of columns and relations of the entity.
     */
    createPropertiesMap(): {
        [name: string]: string | any;
    };
    /**
     * Computes property name of the entity using given PropertyTypeInFunction.
     */
    computePropertyName(nameOrFn: PropertyTypeInFunction<any>): any;
    /**
     * Returns entity id of the given entity.
     */
    getEntityId(entity: any): any;
    /**
     * Checks if column with the given property name exist.
     */
    hasColumnWithPropertyName(propertyName: string): boolean;
    /**
     * Checks if column with the given database name exist.
     */
    hasColumnWithDbName(name: string): boolean;
    /**
     * Checks if relation with the given property name exist.
     */
    hasRelationWithPropertyName(propertyName: string): boolean;
    /**
     * Finds relation with the given property name.
     */
    findRelationWithPropertyName(propertyName: string): RelationMetadata;
    /**
     * Checks if relation with the given name exist.
     */
    hasRelationWithDbName(dbName: string): boolean;
    /**
     * Finds relation with the given name.
     */
    findRelationWithDbName(name: string): RelationMetadata;
    addColumn(column: ColumnMetadata): void;
    extractNonEmptyColumns(object: ObjectLiteral): ColumnMetadata[];
    extractNonEmptySingleValueRelations(object: ObjectLiteral): RelationMetadata[];
    extractNonEmptyMultiValueRelations(object: ObjectLiteral): RelationMetadata[];
    extractExistSingleValueRelations(object: ObjectLiteral): RelationMetadata[];
    extractExistMultiValueRelations(object: ObjectLiteral): RelationMetadata[];
}
