"use strict";
const RelationTypes_1 = require("./types/RelationTypes");
/**
 * Contains all entity metadata.
 */
class EntityMetadata {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(lazyRelationsWrapper, args) {
        this.lazyRelationsWrapper = lazyRelationsWrapper;
        /**
         * Entity's foreign key metadatas.
         */
        this.foreignKeys = [];
        this.namingStrategy = args.namingStrategy;
        this.table = args.tableMetadata;
        this._columns = args.columnMetadatas || [];
        this.relations = args.relationMetadatas || [];
        this.indices = args.indexMetadatas || [];
        this.foreignKeys = args.foreignKeyMetadatas || [];
        this.embeddeds = args.embeddedMetadatas || [];
        this.table.entityMetadata = this;
        this._columns.forEach(column => column.entityMetadata = this);
        this.relations.forEach(relation => relation.entityMetadata = this);
        this.foreignKeys.forEach(foreignKey => foreignKey.entityMetadata = this);
        this.indices.forEach(index => index.entityMetadata = this);
        this.embeddeds.forEach(embedded => embedded.entityMetadata = this);
    }
    // -------------------------------------------------------------------------
    // Accessors
    // -------------------------------------------------------------------------
    /**
     * Entity's name. Equal to entity target class's name if target is set to table, or equals to table name if its set.
     */
    get name() {
        if (!this.table)
            throw new Error("No table target set to the entity metadata.");
        return this.targetName ? this.targetName : this.table.name;
    }
    /**
     * All columns of the entity, including columns that are coming from the embeddeds of this entity.
     */
    get columns() {
        let allColumns = [].concat(this._columns);
        this.embeddeds.forEach(embedded => {
            allColumns = allColumns.concat(embedded.columns);
        });
        return allColumns;
    }
    /**
     * Target class to which this entity metadata is bind.
     */
    get target() {
        return this.table.target;
    }
    /**
     * Gets the name of the target.
     */
    get targetName() {
        if (typeof this.target === "string")
            return this.target;
        if (this.target instanceof Function)
            return this.target.name;
        return "";
    }
    /**
     * Checks if entity has a primary column. All user's entities must have a primary column.
     * Special entity metadatas like for junction tables and closure junction tables don't have a primary column.
     */
    get hasPrimaryColumn() {
        return !!this._columns.find(column => column.isPrimary);
    }
    /**
     * Gets the primary column.
     */
    get primaryColumn() {
        const primaryKey = this._columns.find(column => column.isPrimary);
        if (!primaryKey)
            throw new Error(`Primary key is not set for the ${this.name} entity.`);
        return primaryKey;
    }
    /**
     * Checks if entity has a create date column.
     */
    get hasCreateDateColumn() {
        return !!this._columns.find(column => column.mode === "createDate");
    }
    /**
     * Gets entity column which contains a create date value.
     */
    get createDateColumn() {
        const column = this._columns.find(column => column.mode === "createDate");
        if (!column)
            throw new Error(`CreateDateColumn was not found in entity ${this.name}`);
        return column;
    }
    /**
     * Checks if entity has an update date column.
     */
    get hasUpdateDateColumn() {
        return !!this._columns.find(column => column.mode === "updateDate");
    }
    /**
     * Gets entity column which contains an update date value.
     */
    get updateDateColumn() {
        const column = this._columns.find(column => column.mode === "updateDate");
        if (!column)
            throw new Error(`UpdateDateColumn was not found in entity ${this.name}`);
        return column;
    }
    /**
     * Checks if entity has a version column.
     */
    get hasVersionColumn() {
        return !!this._columns.find(column => column.mode === "version");
    }
    /**
     * Gets entity column which contains an entity version.
     */
    get versionColumn() {
        const column = this._columns.find(column => column.mode === "version");
        if (!column)
            throw new Error(`VersionColumn was not found in entity ${this.name}`);
        return column;
    }
    /**
     * Checks if entity has a tree level column.
     */
    get hasTreeLevelColumn() {
        return !!this._columns.find(column => column.mode === "treeLevel");
    }
    get treeLevelColumn() {
        const column = this._columns.find(column => column.mode === "treeLevel");
        if (!column)
            throw new Error(`TreeLevelColumn was not found in entity ${this.name}`);
        return column;
    }
    /**
     * Gets single (values of which does not contain arrays) relations.
     */
    get singleValueRelations() {
        return this.relations.filter(relation => {
            return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE || relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY;
        });
    }
    /**
     * Gets single (values of which does not contain arrays) relations.
     */
    get multiValueRelations() {
        return this.relations.filter(relation => {
            return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE || relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY;
        });
    }
    /**
     * Gets only one-to-one relations of the entity.
     */
    get oneToOneRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE);
    }
    /**
     * Gets only owner one-to-one relations of the entity.
     */
    get ownerOneToOneRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE && relation.isOwning);
    }
    /**
     * Gets only one-to-many relations of the entity.
     */
    get oneToManyRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY);
    }
    /**
     * Gets only many-to-one relations of the entity.
     */
    get manyToOneRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE);
    }
    /**
     * Gets only many-to-many relations of the entity.
     */
    get manyToManyRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY);
    }
    /**
     * Gets only owner many-to-many relations of the entity.
     */
    get ownerManyToManyRelations() {
        return this.relations.filter(relation => relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY && relation.isOwning);
    }
    /**
     * Gets only owner one-to-one and many-to-one relations.
     */
    get relationsWithJoinColumns() {
        return this.ownerOneToOneRelations.concat(this.manyToOneRelations);
    }
    /**
     * Checks if there is a tree parent relation. Used only in tree-tables.
     */
    get hasTreeParentRelation() {
        return !!this.relations.find(relation => relation.isTreeParent);
    }
    /**
     * Tree parent relation. Used only in tree-tables.
     */
    get treeParentRelation() {
        const relation = this.relations.find(relation => relation.isTreeParent);
        if (!relation)
            throw new Error(`TreeParent relation was not found in entity ${this.name}`);
        return relation;
    }
    /**
     * Checks if there is a tree children relation. Used only in tree-tables.
     */
    get hasTreeChildrenRelation() {
        return !!this.relations.find(relation => relation.isTreeChildren);
    }
    /**
     * Tree children relation. Used only in tree-tables.
     */
    get treeChildrenRelation() {
        const relation = this.relations.find(relation => relation.isTreeChildren);
        if (!relation)
            throw new Error(`TreeParent relation was not found in entity ${this.name}`);
        return relation;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new entity.
     */
    create() {
        // if target is set to a function (e.g. class) that can be created then create it
        if (this.table.target instanceof Function)
            return new this.table.target();
        // otherwise simply return a new empty object
        const newObject = {};
        this.relations
            .filter(relation => relation.isLazy)
            .forEach(relation => this.lazyRelationsWrapper.wrap(newObject, relation));
        return newObject;
    }
    /**
     * Creates an object - map of columns and relations of the entity.
     */
    createPropertiesMap() {
        const entity = {};
        this._columns.forEach(column => entity[column.propertyName] = column.propertyName);
        this.relations.forEach(relation => entity[relation.propertyName] = relation.propertyName);
        return entity;
    }
    /**
     * Computes property name of the entity using given PropertyTypeInFunction.
     */
    computePropertyName(nameOrFn) {
        return typeof nameOrFn === "string" ? nameOrFn : nameOrFn(this.createPropertiesMap());
    }
    /**
     * Returns entity id of the given entity.
     */
    getEntityId(entity) {
        return entity ? entity[this.primaryColumn.propertyName] : undefined;
    }
    /**
     * Checks if column with the given property name exist.
     */
    hasColumnWithPropertyName(propertyName) {
        return !!this._columns.find(column => column.propertyName === propertyName);
    }
    /**
     * Checks if column with the given database name exist.
     */
    hasColumnWithDbName(name) {
        return !!this._columns.find(column => column.name === name);
    }
    /**
     * Checks if relation with the given property name exist.
     */
    hasRelationWithPropertyName(propertyName) {
        return !!this.relations.find(relation => relation.propertyName === propertyName);
    }
    /**
     * Finds relation with the given property name.
     */
    findRelationWithPropertyName(propertyName) {
        const relation = this.relations.find(relation => relation.propertyName === propertyName);
        if (!relation)
            throw new Error(`Relation with property name ${propertyName} in ${this.name} entity was not found.`);
        return relation;
    }
    /**
     * Checks if relation with the given name exist.
     */
    hasRelationWithDbName(dbName) {
        return !!this.relationsWithJoinColumns.find(relation => relation.name === dbName);
    }
    /**
     * Finds relation with the given name.
     */
    findRelationWithDbName(name) {
        const relation = this.relationsWithJoinColumns.find(relation => relation.name === name);
        if (!relation)
            throw new Error(`Relation with name ${name} in ${this.name} entity was not found.`);
        return relation;
    }
    addColumn(column) {
        this._columns.push(column);
        column.entityMetadata = this;
    }
    extractNonEmptyColumns(object) {
        return this.columns.filter(column => !!object[column.propertyName]);
    }
    extractNonEmptySingleValueRelations(object) {
        return this.relations.filter(relation => {
            return (relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE || relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE)
                && !!object[relation.propertyName];
        });
    }
    extractNonEmptyMultiValueRelations(object) {
        return this.relations.filter(relation => {
            return (relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY || relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY)
                && !!object[relation.propertyName];
        });
    }
    extractExistSingleValueRelations(object) {
        return this.relations.filter(relation => {
            return (relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE || relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE)
                && object.hasOwnProperty(relation.propertyName);
        });
    }
    extractExistMultiValueRelations(object) {
        return this.relations.filter(relation => {
            return (relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY || relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY)
                && object.hasOwnProperty(relation.propertyName);
        });
    }
}
exports.EntityMetadata = EntityMetadata;
//# sourceMappingURL=EntityMetadata.js.map