"use strict";
const Repository_1 = require("./Repository");
const Rx = require("rxjs/Rx");
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 * This version of Repository is using rxjs library and Observables instead of promises.
 */
class ReactiveRepository {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(repository) {
        this.repository = repository;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Repository's target is an object (or unique string in the case if entity is loaded from a schema) that is managed
     * by this repository.
     */
    get target() {
        return this.repository.target;
    }
    /**
     * Checks if entity has an id.
     */
    hasId(entity) {
        return this.repository.hasId(entity);
    }
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias) {
        return this.repository.createQueryBuilder(alias);
    }
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create(fromRawEntity) {
        return this.repository.create(fromRawEntity);
    }
    /**
     * Creates entities from a given array of plain javascript objects.
     */
    createMany(copyFromObjects) {
        return this.repository.createMany(copyFromObjects);
    }
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize(object) {
        return Rx.Observable.fromPromise(this.repository.initialize(object));
    }
    /**
     * Merges two entities into one new entity.
     */
    merge(...objects) {
        return this.repository.merge(...objects);
    }
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    persist(entity) {
        return Rx.Observable.fromPromise(this.repository.persist(entity));
    }
    /**
     * Removes a given entity from the database.
     */
    remove(entity) {
        return Rx.Observable.fromPromise(this.repository.remove(entity));
    }
    /**
     * Finds entities that match given conditions.
     */
    find(conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.find(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.find(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.find());
        }
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.findAndCount(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.findAndCount(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.findAndCount());
        }
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.findOne(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.findOne(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.findOne());
        }
    }
    /**
     * Finds entity with given id.
     */
    findOneById(id, options) {
        return Rx.Observable.fromPromise(this.repository.findOneById(id, options));
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        return Rx.Observable.fromPromise(this.repository.query(query));
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        return Rx.Observable.fromPromise(this.repository.transaction(runInTransaction));
    }
    setRelation(relationName, entityId, relatedEntityId) {
        return Rx.Observable.fromPromise(this.repository.setRelation(relationName, entityId, relatedEntityId));
    }
    addToRelation(relationName, entityId, relatedEntityIds) {
        return Rx.Observable.fromPromise(this.repository.addToRelation(relationName, entityId, relatedEntityIds));
    }
    removeFromRelation(relationName, entityId, relatedEntityIds) {
        return Rx.Observable.fromPromise(this.repository.removeFromRelation(relationName, entityId, relatedEntityIds));
    }
    addAndRemoveFromRelation(relation, entityId, addRelatedEntityIds, removeRelatedEntityIds) {
        return Rx.Observable.fromPromise(this.repository.addAndRemoveFromRelation(relation, entityId, addRelatedEntityIds, removeRelatedEntityIds));
    }
    /**
     * Removes entity with the given id.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeById(id) {
        return Rx.Observable.fromPromise(this.repository.removeById(id));
    }
    /**
     * Removes all entities with the given ids.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeByIds(ids) {
        return Rx.Observable.fromPromise(this.repository.removeByIds(ids));
    }
    // -------------------------------------------------------------------------
    // Static Methods
    // -------------------------------------------------------------------------
    static ownsMetadata(reactiveRepository, metadata) {
        return Repository_1.Repository.ownsMetadata(reactiveRepository.repository, metadata);
    }
}
exports.ReactiveRepository = ReactiveRepository;
//# sourceMappingURL=ReactiveRepository.js.map