"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const QueryBuilder_1 = require("../query-builder/QueryBuilder");
const PlainObjectToNewEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToNewEntityTransformer");
const PlainObjectToDatabaseEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToDatabaseEntityTransformer");
const EntityPersistOperationsBuilder_1 = require("../persistment/EntityPersistOperationsBuilder");
const PersistOperationExecutor_1 = require("../persistment/PersistOperationExecutor");
const FindOptions_1 = require("./FindOptions");
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 */
class Repository {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection, broadcaster, entityMetadatas, metadata) {
        this.connection = connection;
        this.broadcaster = broadcaster;
        this.entityMetadatas = entityMetadatas;
        this.metadata = metadata;
        this.driver = connection.driver;
        this.persistOperationExecutor = new PersistOperationExecutor_1.PersistOperationExecutor(connection.driver, entityMetadatas, this.broadcaster);
        this.entityPersistOperationBuilder = new EntityPersistOperationsBuilder_1.EntityPersistOperationBuilder(entityMetadatas);
        this.plainObjectToEntityTransformer = new PlainObjectToNewEntityTransformer_1.PlainObjectToNewEntityTransformer();
        this.plainObjectToDatabaseEntityTransformer = new PlainObjectToDatabaseEntityTransformer_1.PlainObjectToDatabaseEntityTransformer();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Repository's target is an object (or unique string in the case if entity is loaded from a schema) that is managed
     * by this repository.
     */
    get target() {
        return this.metadata.target;
    }
    /**
     * Checks if entity has an id.
     */
    hasId(entity) {
        const columnName = this.metadata.primaryColumn.propertyName;
        return !!(entity &&
            entity.hasOwnProperty(columnName) &&
            entity[columnName] !== null &&
            entity[columnName] !== undefined &&
            entity[columnName] !== "");
    }
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias) {
        return new QueryBuilder_1.QueryBuilder(this.driver, this.entityMetadatas, this.broadcaster)
            .select(alias)
            .from(this.metadata.target, alias);
    }
    /**
     * Creates a new entity. If plainObject is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    create(plainObject) {
        const newEntity = this.metadata.create();
        if (plainObject)
            this.plainObjectToEntityTransformer.transform(newEntity, plainObject, this.metadata);
        return newEntity;
    }
    /**
     * Creates entities from a given array of plain javascript objects.
     */
    createMany(plainObjects) {
        return plainObjects.map(object => this.create(object));
    }
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    initialize(object) {
        const queryBuilder = this.createQueryBuilder(this.metadata.table.name);
        return this.plainObjectToDatabaseEntityTransformer.transform(object, this.metadata, queryBuilder);
    }
    /**
     * Merges multiple entities (or entity-like objects) into one new entity.
     */
    merge(...objects) {
        const newEntity = this.create();
        objects.forEach(object => this.plainObjectToEntityTransformer.transform(newEntity, object, this.metadata));
        return newEntity;
        /*const comparator = new Comporator(metadata, entity, object, ComparationMode.NON_EMPTY);
         comparator.columns.forEach(column => {
         entity[column.propertyName] = object[column.propertyName];
         });
         comparator.singleValueRelations.forEach(relation => {
         entity[relation.propertyName] = this.mergeEntityWithPlainObject(relation.inverseEntityMetadata, entity[relation.propertyName], object[relation.propertyName]);
         });
         comparator.multiValueRelations.forEach(relation => {
         entity[relation.propertyName] = (object[relation.propertyName] as ObjectLiteral[]).map(subObject => {
         return this.mergeEntityWithPlainObject(relation.inverseEntityMetadata, entity[relation.propertyName], subObject);
         });
         });
         // comparator.applyToRelationItems(() => {});
         */
        /*metadata.extractNonEmptyColumns(object).forEach(column => {
            entity[column.propertyName] = object[column.propertyName];
        });

        metadata.extractExistSingleValueRelations(object).forEach(relation => {
            if (entity[relation.propertyName] instanceof Object) {
                entity[relation.propertyName] = this.mergeEntityWithPlainObject(relation.inverseEntityMetadata, entity[relation.propertyName], object[relation.propertyName]);
            } else {
                entity[relation.propertyName] = object[relation.propertyName]; // todo: do we really need to do it? - probably yes for null and undefined values, but what about others
            }
        });

        metadata.extractExistMultiValueRelations(object).forEach(relation => {
            entity[relation.propertyName] = (object[relation.propertyName] as ObjectLiteral[]).map(subObject => {
                if (entity[relation.propertyName] instanceof Object) {
                    return this.mergeEntityWithPlainObject(relation.inverseEntityMetadata, entity[relation.propertyName], subObject);
                } else {
                    entity[relation.propertyName] = object[relation.propertyName]; // todo: do we really need to do it? - probably yes for null and undefined values, but what about others
                }
            });
        });*/
    }
    /**
     * Persists many objects in the same time.
     */
    persistMany(...entities) {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.all(entities.map(entity => this.persist(entity)));
        });
    }
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    persist(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.resolve(); // resolve is required because need to wait until lazy relations loaded
            const allPersistedEntities = yield this.extractObjectsById(entity, this.metadata);
            let loadedDbEntity = null;
            if (this.hasId(entity))
                loadedDbEntity = yield this.initialize(entity);
            let entityWithIds = [];
            if (loadedDbEntity)
                entityWithIds = yield this.extractObjectsById(loadedDbEntity, this.metadata);
            // need to find db entities that were not loaded by initialize method
            const allDbEntities = yield this.findNotLoadedIds(entityWithIds, allPersistedEntities);
            const persistedEntity = {
                id: this.metadata.getEntityId(entity),
                entityTarget: this.metadata.target,
                entity: entity
            };
            const dbEntity = {
                id: this.metadata.getEntityId(loadedDbEntity),
                entityTarget: this.metadata.target,
                entity: loadedDbEntity
            };
            const persistOperation = yield this.entityPersistOperationBuilder.buildFullPersistment(this.metadata, dbEntity, persistedEntity, allDbEntities, allPersistedEntities);
            yield this.persistOperationExecutor.executePersistOperation(persistOperation);
            return entity;
        });
    }
    /**
     * Removes a given entity from the database.
     */
    remove(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            const dbEntity = yield this.initialize(entity);
            entity[this.metadata.primaryColumn.name] = undefined;
            const [dbEntities, allPersistedEntities] = yield Promise.all([
                this.extractObjectsById(dbEntity, this.metadata),
                this.extractObjectsById(entity, this.metadata)
            ]);
            const entityWithId = {
                id: this.metadata.getEntityId(entity),
                entityTarget: this.metadata.target,
                entity: entity
            };
            const dbEntityWithId = {
                id: this.metadata.getEntityId(dbEntity),
                entityTarget: this.metadata.target,
                entity: dbEntity
            };
            const persistOperation = this.entityPersistOperationBuilder.buildOnlyRemovement(this.metadata, dbEntityWithId, entityWithId, dbEntities, allPersistedEntities);
            yield this.persistOperationExecutor.executePersistOperation(persistOperation);
            return entity;
        });
    }
    /**
     * Finds entities that match given conditions.
     */
    find(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder(conditionsOrFindOptions, options)
                .getResults();
        });
    }
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const qb = this.createFindQueryBuilder(conditionsOrFindOptions, options);
            return qb.getResultsAndCount();
        });
    }
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditionsOrFindOptions, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder(conditionsOrFindOptions, options)
                .getSingleResult();
        });
    }
    /**
     * Finds entity with given id.
     */
    findOneById(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createFindQueryBuilder({ [this.metadata.primaryColumn.name]: id }, options)
                .getSingleResult();
        });
    }
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.driver.query(query);
        });
    }
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            let runInTransactionResult;
            return this.driver
                .beginTransaction()
                .then(() => runInTransaction())
                .then(result => {
                runInTransactionResult = result;
                return this.driver.commitTransaction();
            })
                .catch(err => {
                return this.driver.rollbackTransaction()
                    .then(() => {
                    throw err;
                })
                    .catch(() => {
                    throw err;
                });
            })
                .then(() => runInTransactionResult);
        });
    }
    setRelation(relationName, entityId, relatedEntityId) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            // if (relation.isManyToMany || relation.isOneToMany || relation.isOneToOneNotOwner)
            //     throw new Error(`Only many-to-one and one-to-one with join column are supported for this operation. ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            if (relation.isManyToMany)
                throw new Error(`Many-to-many relation is not supported for this operation. Use #addToRelation method for many-to-many relations.`);
            let table, values = {}, conditions = {};
            if (relation.isOwning) {
                table = relation.entityMetadata.table.name;
                values[relation.name] = relatedEntityId;
                conditions[relation.joinColumn.referencedColumn.name] = entityId;
            }
            else {
                table = relation.inverseEntityMetadata.table.name;
                values[relation.inverseRelation.name] = relatedEntityId;
                conditions[relation.inverseRelation.joinColumn.referencedColumn.name] = entityId;
            }
            return this.driver.update(table, values, conditions).then(() => { });
        });
    }
    setInverseRelation(relationName, relatedEntityId, entityId) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            // if (relation.isManyToMany || relation.isOneToMany || relation.isOneToOneNotOwner)
            //     throw new Error(`Only many-to-one and one-to-one with join column are supported for this operation. ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            if (relation.isManyToMany)
                throw new Error(`Many-to-many relation is not supported for this operation. Use #addToRelation method for many-to-many relations.`);
            let table, values = {}, conditions = {};
            if (relation.isOwning) {
                table = relation.inverseEntityMetadata.table.name;
                values[relation.inverseRelation.name] = relatedEntityId;
                conditions[relation.inverseRelation.joinColumn.referencedColumn.name] = entityId;
            }
            else {
                table = relation.entityMetadata.table.name;
                values[relation.name] = relatedEntityId;
                conditions[relation.joinColumn.referencedColumn.name] = entityId;
            }
            return this.driver.update(table, values, conditions).then(() => { });
        });
    }
    addToRelation(relationName, entityId, relatedEntityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            if (!relation.isManyToMany)
                throw new Error(`Only many-to-many relation supported for this operation. However ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            const insertPromises = relatedEntityIds.map(relatedEntityId => {
                const values = {};
                if (relation.isOwning) {
                    values[relation.junctionEntityMetadata.columns[0].name] = entityId;
                    values[relation.junctionEntityMetadata.columns[1].name] = relatedEntityId;
                }
                else {
                    values[relation.junctionEntityMetadata.columns[1].name] = entityId;
                    values[relation.junctionEntityMetadata.columns[0].name] = relatedEntityId;
                }
                return this.driver.insert(relation.junctionEntityMetadata.table.name, values);
            });
            return Promise.all(insertPromises).then(() => { });
        });
    }
    addToInverseRelation(relationName, relatedEntityId, entityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            if (!relation.isManyToMany)
                throw new Error(`Only many-to-many relation supported for this operation. However ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            const insertPromises = entityIds.map(entityId => {
                const values = {};
                if (relation.isOwning) {
                    values[relation.junctionEntityMetadata.columns[0].name] = entityId;
                    values[relation.junctionEntityMetadata.columns[1].name] = relatedEntityId;
                }
                else {
                    values[relation.junctionEntityMetadata.columns[1].name] = entityId;
                    values[relation.junctionEntityMetadata.columns[0].name] = relatedEntityId;
                }
                return this.driver.insert(relation.junctionEntityMetadata.table.name, values);
            });
            return Promise.all(insertPromises).then(() => { });
        });
    }
    removeFromRelation(relationName, entityId, relatedEntityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            if (!relation.isManyToMany)
                throw new Error(`Only many-to-many relation supported for this operation. However ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            // check if given relation entity ids is empty - then nothing to do here (otherwise next code will remove all ids)
            if (!relatedEntityIds || !relatedEntityIds.length)
                return Promise.resolve();
            const qb = this.createQueryBuilder("junctionEntity")
                .delete(relation.junctionEntityMetadata.table.name);
            const firstColumnName = relation.isOwning ? relation.junctionEntityMetadata.columns[0].name : relation.junctionEntityMetadata.columns[1].name;
            const secondColumnName = relation.isOwning ? relation.junctionEntityMetadata.columns[1].name : relation.junctionEntityMetadata.columns[0].name;
            relatedEntityIds.forEach((relatedEntityId, index) => {
                qb.orWhere(`(${firstColumnName}=:entityId AND ${secondColumnName}=:relatedEntity_${index})`)
                    .setParameter("relatedEntity_" + index, relatedEntityId);
            });
            return qb
                .setParameter("entityId", entityId)
                .execute()
                .then(() => { });
        });
    }
    removeFromInverseRelation(relationName, relatedEntityId, entityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const propertyName = this.metadata.computePropertyName(relationName);
            if (!this.metadata.hasRelationWithPropertyName(propertyName))
                throw new Error(`Relation ${propertyName} was not found in the ${this.metadata.name} entity.`);
            const relation = this.metadata.findRelationWithPropertyName(propertyName);
            if (!relation.isManyToMany)
                throw new Error(`Only many-to-many relation supported for this operation. However ${this.metadata.name}#${propertyName} relation type is ${relation.relationType}`);
            // check if given entity ids is empty - then nothing to do here (otherwise next code will remove all ids)
            if (!entityIds || !entityIds.length)
                return Promise.resolve();
            const qb = this.createQueryBuilder("junctionEntity")
                .delete(relation.junctionEntityMetadata.table.name);
            const firstColumnName = relation.isOwning ? relation.junctionEntityMetadata.columns[1].name : relation.junctionEntityMetadata.columns[0].name;
            const secondColumnName = relation.isOwning ? relation.junctionEntityMetadata.columns[0].name : relation.junctionEntityMetadata.columns[1].name;
            entityIds.forEach((entityId, index) => {
                qb.orWhere(`(${firstColumnName}=:relatedEntityId AND ${secondColumnName}=:entity_${index})`)
                    .setParameter("entity_" + index, entityId);
            });
            yield qb.setParameter("relatedEntityId", relatedEntityId).execute();
        });
    }
    addAndRemoveFromRelation(relation, entityId, addRelatedEntityIds, removeRelatedEntityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                this.addToRelation(relation, entityId, addRelatedEntityIds),
                this.removeFromRelation(relation, entityId, removeRelatedEntityIds)
            ]);
        });
    }
    addAndRemoveFromInverseRelation(relation, relatedEntityId, addEntityIds, removeEntityIds) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                this.addToInverseRelation(relation, relatedEntityId, addEntityIds),
                this.removeFromInverseRelation(relation, relatedEntityId, removeEntityIds)
            ]);
        });
    }
    /**
     * Removes entity with the given id.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const alias = this.metadata.table.name;
            yield this.createQueryBuilder(alias)
                .delete()
                .where(alias + "." + this.metadata.primaryColumn.propertyName + "=:id", { id: id })
                .execute();
        });
    }
    /**
     * Removes all entities with the given ids.
     * Note that event listeners and event subscribers won't work (and will not send any events) when using this operation.
     */
    removeByIds(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const alias = this.metadata.table.name;
            yield this.createQueryBuilder(alias)
                .delete()
                .where(alias + "." + this.metadata.primaryColumn.propertyName + " IN (:ids)", { ids: ids })
                .execute();
        });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    createFindQueryBuilder(conditionsOrFindOptions, options) {
        const findOptions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? conditionsOrFindOptions : options;
        const conditions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? undefined : conditionsOrFindOptions;
        const alias = findOptions ? findOptions.alias : this.metadata.table.name;
        const qb = this.createQueryBuilder(alias);
        if (findOptions) {
            FindOptions_1.FindOptionsUtils.applyOptionsToQueryBuilder(qb, findOptions);
        }
        if (conditions) {
            Object.keys(conditions).forEach(key => {
                const name = key.indexOf(".") === -1 ? alias + "." + key : key;
                qb.andWhere(name + "=:" + key);
            });
            qb.addParameters(conditions);
        }
        return qb;
    }
    /**
     * When ORM loads dbEntity it uses joins to load all entity dependencies. However when dbEntity is newly persisted
     * to the db, but uses already exist in the db relational entities, those entities cannot be loaded, and will
     * absent in dbEntities. To fix it, we need to go throw all persistedEntities we have, find out those which have
     * ids, check if we did not load them yet and try to load them. This algorithm will make sure that all dbEntities
     * are loaded. Further it will help insert operations to work correctly.
     */
    findNotLoadedIds(dbEntities, persistedEntities) {
        const missingDbEntitiesLoad = persistedEntities
            .filter(entityWithId => entityWithId.id !== null && entityWithId.id !== undefined)
            .filter(entityWithId => !dbEntities.find(dbEntity => dbEntity.entityTarget === entityWithId.entityTarget && dbEntity.id === entityWithId.id))
            .map(entityWithId => {
            const metadata = this.entityMetadatas.findByTarget(entityWithId.entityTarget);
            const repository = this.connection.getRepository(entityWithId.entityTarget); // todo: fix type
            return repository.findOneById(entityWithId.id).then(loadedEntity => {
                if (!loadedEntity)
                    return undefined;
                return {
                    id: loadedEntity[metadata.primaryColumn.name],
                    entityTarget: metadata.target,
                    entity: loadedEntity
                };
            });
        });
        return Promise.all(missingDbEntitiesLoad).then(missingDbEntities => {
            return dbEntities.concat(missingDbEntities.filter(dbEntity => !!dbEntity));
        });
    }
    /**
     * Extracts unique objects from given entity and all its downside relations.
     */
    extractObjectsById(entity, metadata, entityWithIds = []) {
        const promises = metadata.relations.map(relation => {
            const relMetadata = relation.inverseEntityMetadata;
            const value = (entity[relation.propertyName] instanceof Promise && relation.isLazy) ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
            if (!value)
                return undefined;
            if (value instanceof Array) {
                const subPromises = value.map((subEntity) => {
                    return this.extractObjectsById(subEntity, relMetadata, entityWithIds);
                });
                return Promise.all(subPromises);
            }
            else {
                return this.extractObjectsById(value, relMetadata, entityWithIds);
            }
        });
        return Promise.all(promises.filter(result => !!result)).then(() => {
            if (!entityWithIds.find(entityWithId => entityWithId.entity === entity)) {
                entityWithIds.push({
                    id: entity[metadata.primaryColumn.name],
                    entityTarget: metadata.target,
                    entity: entity
                });
            }
            return entityWithIds;
        });
    }
    // -------------------------------------------------------------------------
    // Static Methods
    // -------------------------------------------------------------------------
    /**
     * Checks if given repository owns given metadata.
     */
    static ownsMetadata(repository, metadata) {
        return repository.metadata === metadata;
    }
}
exports.Repository = Repository;
//# sourceMappingURL=Repository.js.map