"use strict";
const SchemaBuilder_1 = require("./SchemaBuilder");
const ForeignKeyMetadata_1 = require("../metadata/ForeignKeyMetadata");
/**
 * @internal
 */
class PostgresSchemaBuilder extends SchemaBuilder_1.SchemaBuilder {
    constructor(driver) {
        super();
        this.driver = driver;
    }
    getChangedColumns(tableName, columns) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_CATALOG = '${this.driver.db}'` +
            ` AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => {
            return columns.filter(column => {
                const dbData = results.find(result => result.column_name === column.name);
                if (!dbData)
                    return false;
                const newType = this.normalizeType(column);
                const isNullable = column.isNullable === true ? "YES" : "NO";
                let columnType = dbData.data_type.toLowerCase();
                if (dbData.character_maximum_length) {
                    columnType += "(" + dbData.character_maximum_length + ")";
                }
                // const hasDbColumnAutoIncrement = dbData.EXTRA.indexOf("auto_increment") !== -1;
                // const hasDbColumnPrimaryIndex = dbData.column_key.indexOf("PRI") !== -1;
                return columnType !== newType ||
                    // dbData.COLUMN_COMMENT !== column.comment ||
                    dbData.is_nullable !== isNullable; // ||
                // hasDbColumnAutoIncrement !== column.isGenerated ||
                // hasDbColumnPrimaryIndex !== column.isPrimary;
            }).map(column => {
                // const dbData = results.find(result => result.column_name === column.name);
                // const hasDbColumnPrimaryIndex = dbData.column_key.indexOf("PRI") !== -1;
                return { columnName: column.name /*, hasPrimaryKey: hasDbColumnPrimaryIndex*/ };
            });
        });
    }
    checkIfTableExist(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG = '${this.driver.db}' AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => !!(results && results.length));
    }
    addColumnQuery(tableName, column) {
        const sql = `ALTER TABLE ${tableName} ADD ${this.buildCreateColumnSql(column, false)}`;
        return this.query(sql).then(() => { });
    }
    dropColumnQuery(tableName, columnName) {
        const sql = `ALTER TABLE ${tableName} DROP ${columnName}`;
        return this.query(sql).then(() => { });
    }
    addForeignKeyQuery(foreignKey) {
        let sql = `ALTER TABLE ${foreignKey.tableName} ADD CONSTRAINT \`${foreignKey.name}\` ` +
            `FOREIGN KEY (${foreignKey.columnNames.join(", ")}) ` +
            `REFERENCES ${foreignKey.referencedTable.name}(${foreignKey.referencedColumnNames.join(",")})`;
        if (foreignKey.onDelete)
            sql += " ON DELETE " + foreignKey.onDelete;
        return this.query(sql).then(() => { });
    }
    dropForeignKeyQuery(tableNameOrForeignKey, foreignKeyName) {
        let tableName = tableNameOrForeignKey;
        if (tableNameOrForeignKey instanceof ForeignKeyMetadata_1.ForeignKeyMetadata) {
            tableName = tableNameOrForeignKey.tableName;
            foreignKeyName = tableNameOrForeignKey.name;
        }
        const sql = `ALTER TABLE ${tableName} DROP FOREIGN KEY \`${foreignKeyName}\``;
        return this.query(sql).then(() => { });
    }
    getTableForeignQuery(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.table_constraints WHERE TABLE_CATALOG = '${this.driver.db}' `
            + `AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE='FOREIGN KEY'`;
        return this.query(sql).then(results => results.map(result => result.CONSTRAINT_NAME));
    }
    getTableUniqueKeysQuery(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_CATALOG = '${this.driver.db}' ` +
            `AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE = 'UNIQUE'`;
        return this.query(sql).then(results => results.map(result => result.CONSTRAINT_NAME));
    }
    getTableIndicesQuery(tableName) {
        const sql = `SHOW INDEX FROM ${tableName}`;
        return this.query(sql).then(results => {
            // exclude foreign keys
            return this.getTableForeignQuery(tableName).then(foreignKeys => {
                return results
                    .filter(result => result.Key_name !== "PRIMARY" && foreignKeys.indexOf(result.Key_name) === -1)
                    .map(result => ({
                    key: result.Key_name,
                    sequence: result.Seq_in_index,
                    column: result.Column_name
                }));
            });
        });
    }
    getPrimaryConstraintName(tableName) {
        const sql = `SELECT * FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_CATALOG = '${this.driver.db}'`
            + ` AND TABLE_NAME = '${tableName}' AND CONSTRAINT_TYPE = 'PRIMARY KEY'`;
        return this.query(sql).then(results => results && results.length ? results[0].CONSTRAINT_NAME : undefined);
    }
    dropIndex(tableName, indexName) {
        const sql = `ALTER TABLE ${tableName} DROP INDEX \`${indexName}\``;
        return this.query(sql).then(() => { });
    }
    createIndex(tableName, index) {
        const sql = `CREATE ${index.isUnique ? "UNIQUE" : ""} INDEX \`${index.name}\` ON ${tableName}(${index.columns.join(", ")})`;
        return this.query(sql).then(() => { });
    }
    addUniqueKey(tableName, columnName, keyName) {
        const sql = `ALTER TABLE ${tableName} ADD CONSTRAINT ${keyName} UNIQUE (${columnName})`;
        return this.query(sql).then(() => { });
    }
    getTableColumns(tableName) {
        const sql = `SELECT column_name FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_CATALOG = '${this.driver.db}'` +
            ` AND TABLE_NAME = '${tableName}'`;
        return this.query(sql).then(results => results.map(result => result.column_name));
    }
    changeColumnQuery(tableName, columnName, newColumn, skipPrimary = false) {
        const sql = `ALTER TABLE ${tableName} CHANGE ${columnName} ${this.buildCreateColumnSql(newColumn, skipPrimary)}`;
        return this.query(sql).then(() => { });
    }
    createTableQuery(table, columns) {
        const columnDefinitions = columns.map(column => this.buildCreateColumnSql(column, false)).join(", ");
        const sql = `CREATE TABLE "${table.name}" (${columnDefinitions})`;
        return this.query(sql).then(() => { });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    query(sql) {
        return this.driver.query(sql);
    }
    buildCreateColumnSql(column, skipPrimary) {
        let c = column.name + " " + this.normalizeType(column);
        if (column.isNullable !== true)
            c += " NOT NULL";
        if (column.isPrimary === true && !skipPrimary)
            c += " PRIMARY KEY";
        // if (column.isGenerated === true) // don't use skipPrimary here since updates can update already exist primary without auto inc.
        //     c += " AUTO_INCREMENT";
        // TODO: implement auto increment
        if (column.comment)
            c += " COMMENT '" + column.comment + "'";
        if (column.columnDefinition)
            c += " " + column.columnDefinition;
        return c;
    }
    normalizeType(column) {
        let realType = "";
        if (typeof column.type === "string") {
            realType = column.type.toLowerCase();
        }
        else if (typeof column.type === "object" && column.type.name && typeof column.type.name === "string") {
            realType = column.type.toLowerCase();
        }
        switch (realType) {
            case "string":
                return "character varying(" + (column.length ? column.length : 255) + ")";
            case "text":
                return "text";
            case "boolean":
                return "boolean";
            case "integer":
            case "int":
                return "integer";
            case "smallint":
                return "smallint";
            case "bigint":
                return "bigint";
            case "float":
                return "real";
            case "double":
            case "number":
                return "double precision";
            case "decimal":
                if (column.precision && column.scale) {
                    return `decimal(${column.precision},${column.scale})`;
                }
                else if (column.scale) {
                    return `decimal(${column.scale})`;
                }
                else if (column.precision) {
                    return `decimal(${column.precision})`;
                }
                else {
                    return "decimal";
                }
            case "date":
                return "date";
            case "time":
                return "time";
            case "datetime":
                return "timestamp";
            case "json":
                return "text";
            case "simple_array":
                return column.length ? "character varying(" + column.length + ")" : "text";
        }
        throw new Error("Specified type (" + column.type + ") is not supported by current driver.");
    }
}
exports.PostgresSchemaBuilder = PostgresSchemaBuilder;
//# sourceMappingURL=PostgresSchemaBuilder.js.map